from django.db import models
try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.conf import settings

from model_utils.models import TimeStampedModel


class Interaction(TimeStampedModel):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        related_name='interactions',
        on_delete=models.CASCADE,
    )
    object_id = models.IntegerField(db_index=True, null=True)
    content_type = models.ForeignKey(
        ContentType,
        null=True,
        related_name='interaction_objects',
        on_delete=models.CASCADE,
    )
    target_object = GenericForeignKey()

    ratings = models.ManyToManyField(
        'OverallRating',
        related_name='interactions')
    rating = models.FloatField(null=True)

    def __str__(self):
        return '{} - {}'.format(self.user, self.target_object)

    def update(self):
        r = self.ratings.all().aggregate(r=models.Avg('rating'))['r'] or 0
        self.rating = round(r, 1)
        self.save()
        self.target_object.stats.get('rating').set(self.rating)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.12 on 2019-03-26 17:19
from __future__ import unicode_literals

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import model_utils.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Interaction',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('object_id', models.IntegerField(db_index=True, null=True)),
                ('rating', models.FloatField(null=True)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                   related_name='interaction_objects', to='contenttypes.ContentType')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='OverallRating',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('rating', models.FloatField(null=True)),
                ('category', models.CharField(blank=True, choices=[('ticket-management-advisor', 'Ticket Management - Advisor'), ('ticket-management-user', 'Ticket Management - User'), (
                    'ticket-advisor', 'Advisor in ticket'), ('answer-advisor', 'Advisor in questions'), ('step-feedback', 'Step Feedback')], max_length=250)),
                ('object_id', models.IntegerField(db_index=True, null=True)),
                ('context_object_id', models.IntegerField(db_index=True, null=True)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                   related_name='overall_objects', to='contenttypes.ContentType')),
                ('context_content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                           related_name='overall_contexts', to='contenttypes.ContentType')),
            ],
        ),
        migrations.CreateModel(
            name='Rating',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('rating', models.IntegerField()),
                ('comment', models.TextField(blank=True, null=True)),
                ('category', models.CharField(blank=True, choices=[('ticket-management-advisor', 'Ticket Management - Advisor'), ('ticket-management-user', 'Ticket Management - User'), (
                    'ticket-advisor', 'Advisor in ticket'), ('answer-advisor', 'Advisor in questions'), ('step-feedback', 'Step Feedback')], max_length=250)),
                ('object_id', models.IntegerField(db_index=True, null=True)),
                ('context_object_id', models.IntegerField(db_index=True, null=True)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                   related_name='rating_objects', to='contenttypes.ContentType')),
                ('context_content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                           related_name='rating_contexts', to='contenttypes.ContentType')),
                ('overall_rating', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                     related_name='ratings', to='ratings.OverallRating')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='SkipRating',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(
                    default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('object_id', models.IntegerField(db_index=True, null=True)),
                ('comment', models.TextField(blank=True, null=True)),
                ('category', models.CharField(blank=True, choices=[('ticket-management-advisor', 'Ticket Management - Advisor'), ('ticket-management-user', 'Ticket Management - User'), (
                    'ticket-advisor', 'Advisor in ticket'), ('answer-advisor', 'Advisor in questions'), ('step-feedback', 'Step Feedback')], max_length=250)),
                ('context_object_id', models.IntegerField(db_index=True, null=True)),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                   related_name='skip_rating_objects', to='contenttypes.ContentType')),
                ('context_content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE,
                                                           related_name='skip_rating_contexts', to='contenttypes.ContentType')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.AddField(
            model_name='interaction',
            name='ratings',
            field=models.ManyToManyField(related_name='interactions', to='ratings.OverallRating'),
        ),
        migrations.AddField(
            model_name='interaction',
            name='user',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE,
                                    related_name='interactions', to=settings.AUTH_USER_MODEL),
        ),
        migrations.AlterUniqueTogether(
            name='skiprating',
            unique_together=set([('object_id', 'content_type', 'user', 'category',
                                  'context_content_type', 'context_object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='rating',
            unique_together=set([('object_id', 'content_type', 'user', 'category',
                                  'context_content_type', 'context_object_id')]),
        ),
        migrations.AlterUniqueTogether(
            name='overallrating',
            unique_together=set([('context_object_id', 'context_content_type',
                                  'category', 'object_id', 'content_type')]),
        ),
    ]



from django.contrib import admin
from . import models
# Register your models here.

admin.site.register(models.Interaction)
admin.site.register(models.OverallRating)
admin.site.register(models.Rating)
admin.site.register(models.SkipRating)



from rest_framework import serializers

from django.contrib.auth import get_user_model

from ...models import OverallRating, Rating


class UserSerializer(serializers.ModelSerializer):
    shortName = serializers.CharField(source='short_name')
    fullName = serializers.CharField(source='full_name')

    class Meta:
        model = get_user_model()
        fields = [
            'pk',
            'shortName',
            'email',
            'fullName',
        ]


class RatingSerializer(serializers.ModelSerializer):
    user = UserSerializer()

    class Meta:
        model = Rating
        fields = [
            'pk',
            'rating',
            'comment',
            'category',
            'user',
        ]


class OverallRatingSerializer(serializers.ModelSerializer):
    ratings = RatingSerializer(many=True)
    userStatus = serializers.SerializerMethodField()
    category = serializers.SerializerMethodField()

    class Meta:
        model = OverallRating
        fields = [
            'pk',
            'rating',
            'category',
            'ratings',
            'userStatus',
        ]

    def get_userStatus(self, obj):
        user = self.context.get('request').user
        return obj.user_status(user)

    def get_category(self, obj):
        return obj.category.upper().replace('-', '_')



# -*- coding: utf-8 -*-
"""
License boilerplate should be used here.
"""
# python 3 imports
from __future__ import absolute_import, unicode_literals

# python imports
import logging

# 3rd. libraries imports
from appconf import AppConf

# django imports
from django.conf import settings  # noqa

logger = logging.getLogger(__name__)


class RatingsConfig(AppConf):
    TICKET_MANAGEMENT_ADVISOR = 'ticket-management-advisor'
    TICKET_MANAGEMENT_USER = 'ticket-management-user'
    TICKET_ADVISOR = 'ticket-advisor'
    ANSWER_ADVISOR = 'answer-advisor'
    STEP_FEEDBACK = 'step-feedback'

    CATEGORIES = (
        (TICKET_MANAGEMENT_ADVISOR, 'Ticket Management - Advisor'),
        (TICKET_MANAGEMENT_USER, 'Ticket Management - User'),
        (TICKET_ADVISOR, 'Advisor in ticket'),
        (ANSWER_ADVISOR, 'Advisor in questions'),
        (STEP_FEEDBACK, 'Step Feedback'),
    )



from django.db import models
try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from model_utils.models import TimeStampedModel

from ..conf import settings


class Rating(TimeStampedModel):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
    )
    rating = models.IntegerField()
    comment = models.TextField(blank=True, null=True)
    category = models.CharField(
        max_length=250,
        blank=True,
        choices=settings.RATINGS_CATEGORIES,
    )
    overall_rating = models.ForeignKey(
        'OverallRating',
        null=True,
        related_name='ratings',
        on_delete=models.CASCADE,
    )

    object_id = models.IntegerField(db_index=True, null=True)
    content_type = models.ForeignKey(
        ContentType,
        null=True,
        related_name='rating_objects',
        on_delete=models.CASCADE,
    )
    content_object = GenericForeignKey()

    context_object_id = models.IntegerField(
        db_index=True, null=True,
    )
    context_content_type = models.ForeignKey(
        ContentType,
        null=True,
        related_name='rating_contexts',
        on_delete=models.CASCADE,
    )
    context_object = GenericForeignKey(
        'context_content_type', 'context_object_id',
    )

    class Meta:
        unique_together = [
            ('object_id', 'content_type', 'user', 'category',
             'context_content_type', 'context_object_id'), ]

    def __str__(self):
        return str(self.rating)

    def clear(self):
        overall = self.overall_rating
        self.delete()
        overall.update()
        return overall.rating

    @classmethod
    def update(
            cls, rating_context, user, rating, category='', comment=None,
            rating_object=None):

        # @@@ Still doing too much in this method
        from .overall_rating import OverallRating
        rated_object = rating_object

        rating_context_ct = ContentType.objects.get_for_model(rating_context)
        rate_objects = cls.objects.filter(
            context_object_id=rating_context.pk,
            context_content_type_id=rating_context_ct.pk,
            user=user,
            category=category,
        )
        if rated_object:
            rated_object_ct = ContentType.objects.get_for_model(rated_object)
            rate_objects = rate_objects.filter(
                object_id=rated_object.pk,
                content_type=rated_object_ct,
            )
        rate_obj = rate_objects.first()

        if rate_obj and rating == 0:
            return rate_obj.clear()

        if rate_obj is None:
            rate_obj = cls.objects.create(
                context_object_id=rating_context.pk,
                context_content_type_id=rating_context_ct.pk,
                user=user,
                category=category,
                rating=rating,
                comment=comment,
            )
            if rated_object:
                rate_obj.content_object = rated_object
                rate_obj.save()

        kwargs = {
            'context_object_id': rating_context.pk,
            'context_content_type_id': rating_context_ct.pk,
            'category': category,
        }
        if rated_object:
            kwargs.update({
                'object_id': rated_object.pk,
                'content_type_id': rated_object_ct.pk,
            })

        overall, _ = OverallRating.objects.get_or_create(**kwargs)
        rate_obj.overall_rating = overall
        rate_obj.rating = rating
        rate_obj.save()

        overall.update()

        return overall.rating



from rest_framework import serializers

from django.contrib.contenttypes.models import ContentType

from ...models import Rating


class RatingSerializer(serializers.ModelSerializer):

    class Meta:
        model = Rating
        fields = [
            'object_id',
            'content_type',
            'rating',
            'comment',
            'category',
            'context_object_id',
            'context_content_type',
        ]

    def create(self, validated_data):
        user = validated_data.get('user')
        context_object_id = validated_data.get('context_object_id')
        context_content_type = validated_data.get('context_content_type')
        content_type = validated_data.get('content_type')
        object_id = validated_data.get('object_id')
        rating_object = None

        ct = ContentType.objects.get_for_id(context_content_type)
        rating_context = ct.get_object_for_this_type(pk=context_object_id)

        if object_id:
            ct = ContentType.objects.get_for_id(content_type)
            rating_object = ct.get_object_for_this_type(pk=object_id)

        return Rating.update(
            rating_context,
            user,
            validated_data.get('rating'),
            validated_data.get('category'),
            validated_data.get('comment'),
            rating_object,
        )



from django.db import models
try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from model_utils.models import TimeStampedModel

from ..conf import settings


class SkipRating(TimeStampedModel):
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE)
    object_id = models.IntegerField(db_index=True, null=True)
    content_type = models.ForeignKey(
        ContentType,
        null=True,
        related_name='skip_rating_objects',
        on_delete=models.CASCADE)
    content_object = GenericForeignKey()
    comment = models.TextField(
        blank=True,
        null=True)
    category = models.CharField(
        max_length=250,
        blank=True,
        choices=settings.RATINGS_CATEGORIES)
    context_object_id = models.IntegerField(
        db_index=True,
        null=True)
    context_content_type = models.ForeignKey(
        ContentType,
        related_name='skip_rating_contexts',
        null=True,
        on_delete=models.CASCADE)
    context_object = GenericForeignKey(
        'context_content_type',
        'context_object_id',
    )

    class Meta:
        unique_together = [
            ('object_id', 'content_type', 'user', 'category', 'context_content_type', 'context_object_id'),
        ]

    def __str__(self):
        return '{} - {}'.format(self.category, self.user)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import re

from setuptools import setup, find_packages


def get_version(*file_paths):
    """Retrieves the version from ratings/__init__.py"""
    filename = os.path.join(os.path.dirname(__file__), *file_paths)
    version_file = open(filename).read()
    version_match = re.search(r"^__version__ = ['\"]([^'\"]*)['\"]",
                              version_file, re.M)
    if version_match:
        return version_match.group(1)
    raise RuntimeError('Unable to find version string.')


version = get_version("ratings", "__init__.py")

readme = open('README.rst').read()
history = open('HISTORY.rst').read().replace('.. :changelog:', '')

setup(
    name='django-ratings-models',
    version=version,
    description='Ratings for your Django models',
    long_description=readme + '\n\n' + history,
    author='marfyl',
    author_email='marfyl.dev@gmail.com',
    url='https://github.com/exolever/django-ratings-models',
    packages=find_packages(),
    include_package_data=True,
    install_requires=['django-appconf', 'django-model-utils', 'djangorestframework'],
    license="MIT",
    zip_safe=False,
    keywords=['python', 'django', 'ratings'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Web Environment',
        'Framework :: Django :: 1.11',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: Implementation :: PyPy',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
    ],
)



from django.db import models
from django.db.models import Avg

try:
    from django.contrib.contenttypes.fields import GenericForeignKey
except ImportError:
    from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType

from model_utils.models import TimeStampedModel

from ..conf import settings
from .rating import Rating
from .skip_rating import SkipRating


class OverallRating(TimeStampedModel):

    rating = models.FloatField(null=True)
    category = models.CharField(
        max_length=250, blank=True,
        choices=settings.RATINGS_CATEGORIES,
    )

    object_id = models.IntegerField(db_index=True, null=True)
    content_type = models.ForeignKey(
        ContentType,
        null=True,
        related_name='overall_objects',
        on_delete=models.CASCADE,
    )
    content_object = GenericForeignKey()

    context_object_id = models.IntegerField(
        db_index=True, null=True,
    )
    context_content_type = models.ForeignKey(
        ContentType,
        null=True,
        related_name='overall_contexts',
        on_delete=models.CASCADE,
    )
    context_object = GenericForeignKey(
        'context_content_type', 'context_object_id',
    )

    class Meta:
        unique_together = [
            ('context_object_id', 'context_content_type', 'category',
             'object_id', 'content_type'),
        ]

    def update(self):
        r = Rating.objects.filter(overall_rating=self).aggregate(r=Avg('rating'))['r'] or 0
        self.rating = r
        self.save()

    def user_status(self, user):
        if self.ratings.filter(user=user).exists():
            return settings.RATINGS_CH_STATUS_RATING
        elif SkipRating.objects.filter(
                category=self.category,
                user=user,
                context_object_id=self.context_object_id,
                context_content_type=self.context_content_type,
        ).exists():
            return settings.RATINGS_CH_STATUS_SKIPPED
        else:
            return settings.RATINGS_CH_STATUS_NOT_RATING



from rest_framework import serializers

from ...models import SkipRating


class SkipRatingSerializer(serializers.ModelSerializer):

    class Meta:
        model = SkipRating
        fields = [
            'object_id',
            'content_type',
            'comment',
            'category',
            'context_object_id',
            'context_content_type',
        ]

    def create(self, validated_data):
        return SkipRating.objects.create(**validated_data)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_ratings
------------

Tests for `ratings` models module.
"""

from django.contrib.auth import get_user_model
from django.test import TestCase

from ratings.models import Rating, OverallRating


class TestRatingsModels(TestCase):

    def test_simple_rating_model(self):
        # Data
        user_from = get_user_model().objects.create()
        overall_rating = OverallRating.objects.create()

        # Do action
        rating = Rating.objects.create(user=user_from, rating=4, overall_rating=overall_rating)

        # Asserts
        self.assertTrue(rating.pk)
        self.assertEqual(rating.user, user_from)

    def test_simple_overall_rating_model_update(self):
        # Data
        user_from = get_user_model().objects.create()
        overall_rating = OverallRating.objects.create()
        rating = Rating.objects.create(user=user_from, rating=4, overall_rating=overall_rating)

        # Do action
        overall_rating.update()

        # Asserts
        self.assertEqual(overall_rating.rating, rating.rating)
